# This file is part of the standard {testthat} test
# temporary directory, which is the default).
if (requireNamespace("testthat", quietly = TRUE)) {
  library(testthat)
  library(teal.picks)

  test_check("teal.picks")
}
