% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/call_utils.R
\name{call_condition_logical}
\alias{call_condition_logical}
\title{\code{logical} variable condition call}
\usage{
call_condition_logical(varname, choice)
}
\arguments{
\item{varname}{(\code{name} or \code{character(1)})
name of the variable}

\item{choice}{(\code{logical(1)})
chosen value}
}
\value{
\code{call}.
}
\description{
Compose \code{logical} variable condition call from inputs.
}
\keyword{internal}
