% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/call_utils.R
\name{calls_combine_by}
\alias{calls_combine_by}
\title{Combine calls by operator}
\usage{
calls_combine_by(operator, calls)
}
\arguments{
\item{operator}{(\code{character(1)} or \code{name}) name / symbol of the operator.}

\item{calls}{(\code{list} of calls) list containing calls to be combined by \code{operator}.}
}
\value{
A combined \code{call}.
}
\description{
Combine list of calls by specific operator.
}
\keyword{internal}
